// --- Que fait ce programme ? ---
/* Affiche sur un LCD les touches tapées sur un clavier matriciel*/
// --- Fonctionnalités utilisées ---
// Utilise un afficheur LCD alphanumérique 4x20 en mode 4 bits 
// Utilise un Clavier matriciel 3x4 (12 touches) 

//**************** Entête déclarative *******
// A ce niveau sont déclarées les librairies, les constantes, les variables...
// --- Inclusion des librairies utilisées ---
#include <LiquidCrystal.h> // Inclusion de la librairie pour afficheur LCD 
#include <Keypad.h> // inclusion de la librairie pour clavier matriciel 

// --- Déclaration des constantes ---
//--- Constantes utilisées avec le clavier 3x4
const byte LIGNES = 4; // 4 lignes
const byte COLONNES = 3; //3 colonnes
// --- constantes des broches ---
const int C3=A2; //declaration constante de broche 
const int C2=A1; //declaration constante de broche 
const int C1=A0; //declaration constante de broche 
const int RS=7; //declaration constante de broche 
const int E=6; //declaration constante de broche 
const int D4=5; //declaration constante de broche 
const int D5=4; //declaration constante de broche 
const int D6=3; //declaration constante de broche 
const int D7=2; //declaration constante de broche 
const int L4=11; //declaration constante de broche 
const int L3=10; //declaration constante de broche 
const int L2=9; //declaration constante de broche 
const int L1=8; //declaration constante de broche 

// --- Déclaration des variables globales ---
//--- Définition des touches 
char touches[LIGNES][COLONNES] = 
{
  {'1','2','3',},
  {'4','5','6',},
  {'7','8','9',},
  {'*','0','#',}
};
// tableaux de lignes et colonnes
byte BrochesLignes[LIGNES] = {L1, L2, L3, L4}; //connexions utilisées pour les broches de lignes du clavier
byte BrochesColonnes[COLONNES] = {C1, C2, C3,}; //connexions utilisées pour les broches de colonnes du clavier

char touche; // variable de stockage valeur touche appuyée

// --- Déclaration des objets utiles pour les fonctionnalités utilisées ---
LiquidCrystal lcd(RS, E, D4, D5, D6, D7);// Création d'un objet LiquidCrystal = initialisation LCD en mode 4 bits 

// création d'un objet keypad = initialisation clavier 
Keypad clavier = Keypad( makeKeymap(touches), BrochesLignes, BrochesColonnes, LIGNES, COLONNES );

//**************** FONCTION SETUP = Code d'initialisation *****
// La fonction setup() est exécutée en premier et 1 seule fois, au démarrage du programme

void setup()   

{ // debut de la fonction setup()

// --- ici instructions à exécuter au démarrage --- 

lcd.begin(20,4); // Initialise le LCD avec 16 colonnes x 2 lignes 

delay(10); // pause rapide pour laisser temps initialisation

// Test du LCD

lcd.print("LCD OK") ; // affiche la chaîne texte - message de test
delay(2000); // pause de 2 secondes

lcd.clear(); // // efface écran et met le curseur en haut à gauche
delay(10); // pour laisser temps effacer écran

} // fin de la fonction setup()

//*************** FONCTION LOOP = Boucle sans fin = coeur du programme *************
// la fonction loop() s'exécute sans fin en boucle aussi longtemps que l'Arduino est sous tension

void loop(){ // debut de la fonction loop()

// --- ici instructions à exécuter par le programme principal --- 

touche = clavier.getKey(); // lecture de la touche appuyée

if (touche != NO_KEY)
  { // si une touche a été frappée -- gestion de la touche appuyée 
    if (touche=='#'or touche=='*') 
    {
    lcd.clear();   // efface écran si appui # ou *
     }
    else //sinon  affiche touche
   {
    lcd.setCursor(0,0);
    lcd.print(touche);
    delay(300); // pause entre 2 appuis
   }   

   } // fin gestion de la touche appuyée 

} // fin de la fonction loop() - le programme recommence au début de la fonction loop //sans fin. Fin du programme
 

