





// --- Fonctionnalités utilisées ---
// Utilise un afficheur LCD alphanumérique 4x20 en mode 4 bits 
// Utilise un Clavier matriciel 3x4 (12 touches) 

//**************** Entête déclarative *******
// A ce niveau sont déclarées les librairies, les constantes, les variables...

// --- Inclusion des librairies utilisées ---

#include <LiquidCrystal.h> // Inclusion de la librairie pour afficheur LCD 
#include <Keypad.h> // inclusion de la librairie pour clavier matriciel 

// --- Déclaration des constantes ---

//--- Constantes utilisées avec le clavier 3x4
const byte LIGNES = 4; // 4 lignes
const byte COLONNES = 3; //3 colonnes


// --- constantes des broches ---

const int C3=A2; //declaration constante de broche 
const int C2=A1; //declaration constante de broche 
const int C1=A0; //declaration constante de broche 
const int RS=7; //declaration constante de broche 
const int E=6; //declaration constante de broche 
const int D4=5; //declaration constante de broche 
const int D5=4; //declaration constante de broche 
const int D6=3; //declaration constante de broche 
const int D7=2; //declaration constante de broche 
const int L4=11; //declaration constante de broche 
const int L3=10; //declaration constante de broche 
const int L2=9; //declaration constante de broche 
const int L1=8; //declaration constante de broche 
const int MARCHE=13;//declaration constante de broche
const int SENS=12;//declaration constante de broche
const int obstacle_ouverture=1;//declaration constante de broche
const int obstacle_fermeture=0;//declaration constante de broche
const int commande_portail=A4;//declaration constante de broche
const int cellule_photoelectrique=A5;//declaration constante de broche
// --- Déclaration des variables globales ---

//--- Définition des touches 
char touches[LIGNES][COLONNES] = 
{
  {'1','2','3',},
  {'4','5','6',},
  {'7','8','9',},
  {'*','0','#',}
};

// tableaux de lignes et colonnes
byte BrochesLignes[LIGNES] = {L1, L2, L3, L4}; //connexions utilisées pour les broches de lignes du clavier
byte BrochesColonnes[COLONNES] = {C1, C2, C3,}; //connexions utilisées pour les broches de colonnes du clavier

// variables globales utiles
char touche=0; // variable de stockage valeur touche appuyée
int valeur=0; // variable pour la valeur numérique de la touche
int compt=0; // variable comptage caractère saisis

String chainesaisie=""; // déclare un objet String vide pour saisie chaine
String codesecret="1234"; // déclare un objet String pour mémoriser le code secret

LiquidCrystal lcd(RS, E, D4, D5, D6, D7);// Création d'un objet LiquidCrystal = initialisation LCD en mode 4 bits 


// création d'un objet keypad = initialisation clavier 
Keypad clavier = Keypad( makeKeymap(touches), BrochesLignes, BrochesColonnes, LIGNES, COLONNES );
// les broches de lignes sont automatiquement configurées en ENTREE avec pullup interne activé 
// les broches de colonnes sont automatiquement configurées en SORTIE


//**************** FONCTION SETUP = Code d'initialisation *****
// La fonction setup() est exécutée en premier et 1 seule fois, au démarrage du programme

void setup()   { // debut de la fonction setup()

// --- ici instructions à exécuter au démarrage --- 
pinMode(MARCHE,OUTPUT);
pinMode(SENS,OUTPUT);
pinMode(commande_portail,INPUT);
pinMode(cellule_photoelectrique,INPUT);
pinMode(obstacle_ouverture,INPUT);
pinMode(obstacle_fermeture,INPUT);
pinMode (commande_portail,INPUT);
pinMode (cellule_photoelectrique,INPUT);
lcd.begin(20,4); // Initialise le LCD avec 16 colonnes x 2 lignes 
lcd.clear();
delay(10); // pause rapide pour laisser temps initialisation

// Test du LCD

lcd.print("LCD OK") ; // affiche la chaîne texte - message de test
delay(1000); // pause de 2 secondes

lcd.clear(); // // efface écran et met le curseur en haut à gauche
delay(10); // pour laisser temps effacer écran

} // fin de la fonction setup()
// ********************************************************************************

//*************** FONCTION LOOP = Boucle sans fin = coeur du programme *************
// la fonction loop() s'exécute sans fin en boucle aussi longtemps que l'Arduino est sous tension

void gestion_du_portail()
  {
    int position_portail = analogRead (A3); //test de l'entrée analogique position capteur
    if (position_portail<900) // test de la position du portail si il n'est pas fermé, on l'ouvre quelle que soit sa position
    {
      OUVERTURE();
    }
    
    else
     {
      FERMETURE();
     }
     
  }
  
  void OUVERTURE ()
  {
    int obstacleouv=digitalRead(obstacle_ouverture);
    int position_portail = analogRead (A3);
    while (position_portail<900 && obstacleouv==0)
      {
        digitalWrite(MARCHE,HIGH);
        digitalWrite(SENS,HIGH);
        position_portail = analogRead (A3);
        obstacleouv=digitalRead(obstacle_ouverture);
         if (obstacleouv==1)
         {
           FERMETURE();
         }
      }
    digitalWrite(MARCHE,LOW);
    digitalWrite(SENS,LOW);  
  }
      
   void FERMETURE ()
   {
     int obstaclefer=digitalRead(obstacle_fermeture);
     int cellule=digitalRead(cellule_photoelectrique);
     int position_portail = analogRead (A3);
     while (position_portail>200 && obstaclefer==0 && cellule==0)
      {
        digitalWrite(MARCHE,HIGH);
        digitalWrite(SENS,LOW);
        position_portail = analogRead (A3);
        obstaclefer=digitalRead(obstacle_fermeture);
        cellule=digitalRead(cellule_photoelectrique);
         if (obstaclefer==1 || cellule==1)
         {
           OUVERTURE();
         }
      }
     digitalWrite(MARCHE,LOW);
     digitalWrite(SENS,LOW);
   }
   
   
void loop(){ // debut de la fonction loop()

// --- ici instructions à exécuter par le programme principal --- 
  
  
  lcd.setCursor(0,0); //1ère colonne - 1ère ligne
  lcd.print("saisir votre code:");
  lcd.setCursor(0,1); //1ère colonne - 2ème ligne
  chainesaisie=saisiecode();
  if (chainesaisie==codesecret)
    {
    lcd.setCursor(0,2); //1ère colonne - 3ème ligne
    lcd.print("CODE CORRECT");
    delay(1000);
    lcd.clear();//efface LCD et se place en 0,0
    delay(10); // courte pause après clear()
    gestion_du_portail();
    }
  else
    {
    lcd.setCursor(0,2); //1ère colonne - 3ème ligne
    lcd.print("CODE INCORRECT");
    delay(1000);
    lcd.clear();//efface LCD et se place en 0,0
    delay(10); // courte pause après clear()
    }
  
  
}

//...........Fonction de saisie d'une chaine au clavier...............
  
String saisiecode()
{
  String chainerenvoyee="";
  
  while(true)//la sortie de la boucle est provoquée par l'appui sur #: break;
  {
  touche = clavier.getKey(); // lecture de la touche appuyée
      if (touche != NO_KEY)
      { // si une touche a été frappée -- gestion de la touche appuyée 
      lcd.print(touche); //affiche le caractère saisi
      valeur=touche-48; //convertit l'ASCII en valeur numérique
        if ((valeur>=0) && (valeur<=9)) //si valeur comprise entre 0 et 9
        {
         chainerenvoyee=chainerenvoyee+touche; //ajoute le caractère au string
        }//fin if
         else if (touche=='#')//si appui sur # = validation
        {
        delay(100); //pause
        break; // sort de la boucle
        }//fin else if
        else
        {
        chainerenvoyee=chainerenvoyee+touche;
        }
      }  //fin if touche appuyée
      int commandeBP=digitalRead(commande_portail); 
      if (commandeBP==1)
        {
          gestion_du_portail();
        }
  }//fin while
 return(chainerenvoyee); //renvoie la chaine saisie
}// fin saisie code

